﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.DataAnalysis {
  [StorableClass]
  [Item("AverageThresholdCalculator", "")]
  public class AverageThresholdCalculator : DiscriminantClassificationWeightCalculator {

    public AverageThresholdCalculator()
      : base() {
    }
    [StorableConstructor]
    protected AverageThresholdCalculator(bool deserializing) : base(deserializing) { }
    protected AverageThresholdCalculator(AverageThresholdCalculator original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new AverageThresholdCalculator(this, cloner);
    }

    protected double[] threshold;
    protected double[] classValues;

    protected override IEnumerable<double> DiscriminantCalculateWeights(IEnumerable<IDiscriminantFunctionClassificationSolution> discriminantSolutions) {
      classValues = discriminantSolutions.First().Model.ClassValues.ToArray();
      var modelThresholds = discriminantSolutions.Select(x => x.Model.Thresholds.ToArray());
      threshold = new double[modelThresholds.First().GetLength(0)];
      for (int i = 0; i < modelThresholds.First().GetLength(0); i++) {
        threshold[i] = modelThresholds.Select(x => x[i]).Average();
      }
      return Enumerable.Repeat<double>(1, discriminantSolutions.Count());
    }

    protected override double GetDiscriminantConfidence(IEnumerable<IDiscriminantFunctionClassificationSolution> solutions, int index, double estimatedClassValue) {
      // only works with binary classification
      if (!classValues.Count().Equals(2))
        return double.NaN;
      Dataset dataset = solutions.First().ProblemData.Dataset;
      IList<double> values = solutions.Select(s => s.Model.GetEstimatedValues(dataset, Enumerable.Repeat(index, 1)).First()).ToList();
      if (values.Count <= 0)
        return double.NaN;
      double avg = values.Average();
      if (estimatedClassValue.Equals(classValues[0])) {
        if (avg < estimatedClassValue)
          return 1;
        else if (avg >= threshold[1])
          return 0;
        else {
          double distance = threshold[1] - classValues[0];
          return (1 / distance) * (threshold[1] - avg);
        }
      } else if (estimatedClassValue.Equals(classValues[1])) {
        if (avg > estimatedClassValue)
          return 1;
        else if (avg <= threshold[1])
          return 0;
        else {
          double distance = classValues[1] - threshold[1];
          return (1 / distance) * (avg - threshold[1]);
        }
      } else
        return double.NaN;
    }
  }
}
